#include "mine.h"

/*!
	@biref コンストラクタ
*/
mine::mine(){
	this->setTitle("マインスイーパ");
	//上ウインドウフレーム幅21px, 左ウインドウフレーム幅 5px
	//下ウインドウフレーム幅5px,  右ウインドウフレーム幅 5px
	//クライアント領域を144 x 144確保する。
	this->setRect(100, 100, 155, 171);
	tile_init();
	tick = 0;
	setTimer(1000);
}

/*!
	@brief タイル群の初期化
*/
void mine::tile_init(){
	bool* bomb_pos = new bool[81];
	make_bombs_position(bomb_pos);
	for(int x=0; x<9; x++){
		for(int y=0; y<9; y++){
			tiles[x][y] = new mine_tile((x << 4), (y << 4));
			
			//とりあえず、そのタイルが爆弾か、爆弾じゃないかを設定
			tiles[x][y]->setTileValue(bomb_pos[((x << 3) + x) + y] ? -1 : 0);
			
		}
	}
	
	//周囲のタイル情報を設定
	for(int x=0; x<9; x++){
		for(int y=0; y<9; y++){
			mine_tile* tile = tiles[x][y];
			if(x == 0){
				if(y == 0){
					tile->setAround(NULL, NULL,
									tiles[x+1][y], tiles[x+1][y+1],
									tiles[x][y+1], NULL,
									NULL, NULL);
				} else if(y == 8){
					tile->setAround(tiles[x][y-1], tiles[x+1][y-1],
									tiles[x+1][y], NULL,
									NULL, NULL,
									NULL, NULL);
				} else {
					tile->setAround(tiles[x][y-1], tiles[x+1][y-1],
									tiles[x+1][y], tiles[x+1][y+1],
									tiles[x][y+1], NULL,
									NULL, NULL);
				}
			} else if(x == 8){
				if(y == 0){
					tile->setAround(NULL, NULL,
									NULL, NULL,
									tiles[x][y+1], tiles[x-1][y+1],
									tiles[x-1][y], NULL);
				} else if(y == 8){
					tile->setAround(tiles[x-1][y], NULL,
									NULL, NULL,
									NULL, NULL,
									tiles[x-1][y], tiles[x-1][y-1]);
				} else {
					tile->setAround(tiles[x][y-1], NULL,
									NULL, NULL,
									tiles[x][y+1], tiles[x-1][y+1],
									tiles[x-1][y], tiles[x-1][y-1]);
				}
			} else {
				if(y == 0){
					tile->setAround(NULL, NULL,
									tiles[x+1][y], tiles[x+1][y+1],
									tiles[x][y+1], tiles[x-1][y+1],
									tiles[x-1][y], NULL);
				} else if(y == 8){
					tile->setAround(tiles[x][y-1], tiles[x+1][y-1],
									tiles[x+1][y], NULL,
									NULL, NULL,
									tiles[x-1][y], tiles[x-1][y-1]);
				} else {
					tile->setAround(tiles[x][y-1], tiles[x+1][y-1],
									tiles[x+1][y], tiles[x+1][y+1],
									tiles[x][y+1], tiles[x-1][y+1],
									tiles[x-1][y], tiles[x-1][y-1]);
				}
			}
			add(tile);
		}
	}
}

/*!
	@brief デストラクタ
*/
mine::~mine(){
	for(int x=0; x<9; x++){
		for(int y=0; y<9; y++){
			delete tiles[x][y];
		}
	}
}

bool mine::is_game_event(Event* evt){
	if(evt->getType() != Event::CUSTOM_EVENT){
		return false;
	}
	
	Control* src = evt->getSource();
	for(int x=0; x<9; x++){
		for(int y=0; y<9; y++){
			if(src == tiles[x][y]){
				return true;
			}
		}
	}
	return false;
}

bool mine::is_gameover(Event* evt){
	if(evt->arg1 == 1){
		return true;
	} else {
		return false;
	}
}

/*!
	@brief イベントプロシージャ
*/
void mine::onEvent(Event* evt){
	if(evt->getType() == Event::TIMER){
		timer_proc();
		return;
	}
	
	if(is_game_event(evt)){
		if(is_gameover(evt)){
			//Alert, MessageBoxの類がないので、printfで代用
			printf("(　ﾟдﾟ)ﾄﾞｶﾝｰ\n");
		} else {
			printf("お(･∀･)め(･∀･)で(･∀･)と(･∀･)う!\n");
			printf("タイム： %d秒\n", tick);
		}
	}
}

int MonaMain(List<char*>* pekoe){
	mine* app = new mine();
	app->run();
	delete app;
	return 0;
}

/*!
	@brief 爆弾の初期位置情報を生成する
*/
void mine::make_bombs_position(bool* ret){
	//初期配置
	for(int i=0; i<81; i++){
		ret[i] = (i < 10) ? true : false;
	}
	
	//シャッフリング
	MonAPI::Random random;
	MonAPI::Date date;
	random.setSeed(date.sec());
	for(int i=0; i<81; i++){
		int swap = random.nextInt() % 81 - i;
		bool tmp = ret[i];
		ret[i] = ret[i+swap];
		ret[i+swap] = tmp;
	}
}

void mine::timer_proc(){
	tick++;
	setTimer(1000);
}
